package week9;


import static java.awt.event.KeyEvent.VK_T;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.Random;

import javax.swing.JFrame;

import com.jogamp.opengl.GL2;
import com.jogamp.opengl.GLAutoDrawable;
import com.jogamp.opengl.GLCapabilities;
import com.jogamp.opengl.GLEventListener;
import com.jogamp.opengl.GLException;
import com.jogamp.opengl.GLProfile;
import com.jogamp.opengl.awt.GLCanvas;
import com.jogamp.opengl.glu.GLU;
import com.jogamp.opengl.util.FPSAnimator;


/**
 * Taken from NeHe Lesson #19a: Fireworks
 */
public class ParticleSystemTornado implements GLEventListener, KeyListener {
   
   //private GLU glu;  // for the GL Utility
   
   private static final int MAX_PARTICLES = 1000; // max number of particles
   private Particle[] particles = new Particle[MAX_PARTICLES];

   private static boolean enabledBurst = false;
   
   // Pull forces in each direction
   private static float gravityY = -0.0008f; // gravity

   // Global speed for all the particles
   private static float speedYGlobal = 0.1f;
   private static float z = -40.0f; //zOffset
   private static float y = -8.0f;   //yOffset
   private static float wiggle = 0;

   // Texture applied over the shape
   private MyTexture myTextures[];
   private String textureFileName = "src/week9/star.bmp";
   private String textureExt = ".bmp";

   
   
   /** The entry main() method */
   public static void main(String[] args) {
	   // initialisation
       GLProfile glp = GLProfile.getDefault();
       GLCapabilities caps = new GLCapabilities(glp);
       
       // create a panel to draw on
       GLCanvas panel = new GLCanvas(caps);
       //GLJPanel panel = new GLJPanel(caps);
       
       // put it in a JFrame
       final JFrame jframe = new JFrame("Particle System");
       jframe.setSize(600, 600);
       jframe.add(panel);
       jframe.setVisible(true);

       // Catch window closing events and quit             
       jframe.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

       ParticleSystemTornado s = new ParticleSystemTornado();
       
       // add a GL Event listener to handle rendering
       panel.addGLEventListener(s);
       panel.addKeyListener(s);
       panel.setFocusable(true);   
       
       // NEW: add an animator to create display events at 60 FPS
       FPSAnimator animator = new FPSAnimator(60,true);
       animator.add(panel);
       animator.start();
   }
   
 

  
   @Override
   public void init(GLAutoDrawable drawable) {
      GL2 gl = drawable.getGL().getGL2();    
                  
      gl.glClearColor(0.0f, 0.0f, 0.0f, 1.0f); 
  
      gl.glHint(GL2.GL_PERSPECTIVE_CORRECTION_HINT, GL2.GL_NICEST); 
      

      // Enable Blending 
      gl.glEnable(GL2.GL_BLEND);      
      //Creates an additive blend, which looks spectacular on a black background
      gl.glBlendFunc(GL2.GL_SRC_ALPHA, GL2.GL_ONE);
     
      
      //Load the texture image
      try {
     	 myTextures = new MyTexture[1];
      	 myTextures[0] = new MyTexture(gl,textureFileName,textureExt,false);
      	//gl.glTexEnvi(GL2.GL_TEXTURE_ENV, GL2.GL_TEXTURE_ENV_MODE, GL2.GL_REPLACE);
      } catch (GLException e) {
         e.printStackTrace();
      } 
     
      gl.glEnable(GL2.GL_TEXTURE_2D);
        
      // Initialize the particles
      for (int i = 0; i < MAX_PARTICLES; i++) {
         particles[i] = new Particle();
      }
   }

   /**
    * Call-back handler for window re-size event. Also called when the drawable is 
    * first set to visible.
    */
   @Override
   public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height) {
      GL2 gl = drawable.getGL().getGL2();  // get the OpenGL 2 graphics context

      if (height == 0) height = 1;   // prevent divide by zero
      float aspect = (float)width / height;

      // Setup perspective projection, with aspect ratio matches viewport
      gl.glMatrixMode(GL2.GL_PROJECTION);  // choose projection matrix
      gl.glLoadIdentity();             // reset projection matrix
      GLU glu = new GLU();
      glu.gluPerspective(45.0, aspect, 0.1, 100.0); // fovy, aspect, zNear, zFar      
      
   }

   
   @Override
   public void display(GLAutoDrawable drawable) {
      GL2 gl = drawable.getGL().getGL2();  // get the OpenGL 2 graphics context
      
     
      
      gl.glClear(GL2.GL_COLOR_BUFFER_BIT | GL2.GL_DEPTH_BUFFER_BIT); // clear color and depth buffers
     
      gl.glMatrixMode(GL2.GL_MODELVIEW);
      gl.glLoadIdentity();  

      // Render the particles
      for (int i = 0; i < MAX_PARTICLES; i++) {
         if (particles[i].active) {
            // Draw the particle using our RGB values
        	
            gl.glColor4f(particles[i].r, particles[i].g, particles[i].b, particles[i].life);
         
         
            
            gl.glBindTexture(GL2.GL_TEXTURE_2D, myTextures[0].getTextureId()); 
        
            
            gl.glBegin(GL2.GL_QUADS);

            float px = (particles[i].y/3 + particles[i].offset)
                      *(float) Math.sin(particles[i].angle)
                      + (float) Math.sin(particles[i].y/5 + wiggle)*3;
            float py = particles[i].y + y;
            float pz = (particles[i].y/3 + particles[i].offset)
                      *(float)Math.cos(particles[i].angle) + z
                      + (float) Math.cos(particles[i].y/5 + wiggle)*3;;

            gl.glTexCoord2d(1, 1);
            gl.glVertex3f(px + 0.5f, py + 0.5f, pz); // Top Right
            gl.glTexCoord2d(0, 1);
            gl.glVertex3f(px - 0.5f, py + 0.5f, pz); // Top Left
            gl.glTexCoord2d(0, 0);
            gl.glVertex3f(px - 0.5f, py - 0.5f, pz); // Bottom Left
            gl.glTexCoord2d(1, 0);
            gl.glVertex3f(px + 0.5f, py - 0.5f, pz); // Bottom Right
            gl.glEnd();

            // Move the particle
            particles[i].angle += 0.05; 
            
            wiggle += 0.00003;
            
            // Apply the gravity force on y-axis
            particles[i].y += 0.1f;
            
            if (particles[i].y >= 20) {
                particles[i].y = 0;
                particles[i].life = 1;
            }
            // Slowly kill it
            particles[i].life -= 0.005;
            
            if (enabledBurst) {
               particles[i].burst();
            }
         }
      }
      if (enabledBurst) enabledBurst = false;
   }

   /** 
    * Called back before the OpenGL context is destroyed. Release resource such as buffers. 
    */
   @Override
   public void dispose(GLAutoDrawable drawable) { }

   // Particle (inner class)
   class Particle {
      boolean active; // always active in this program
      float life;     // how alive it is
      float r, g, b;  // color
      float y, angle;  // position
      float offset;

      private final float[][] colors = {    // rainbow of 12 colors
            { 1.0f, 0.5f, 0.0f }, { 1.0f, 0.75f, 0f },
            { 1f, 1f, 0f }, { 0.5f, 0.5f, 0f }, { 0.75f, 0.5f, 0f },
            { 1.0f, 0.5f, 0f } };

      private Random rand = new Random();

      // Constructor
      public Particle() {
         active = true;
         burst();
      }

      public void burst() {
         // Set the initial position
         y = rand.nextFloat()*20;
         
         // Generate a random speed and direction in polar coordinate, then resolve
         // them into x and y.
         angle = (float)Math.toRadians(rand.nextInt(360));
         offset = rand.nextFloat()*2 - 0.5f;

         int colorIndex = rand.nextInt(colors.length);
         // Pick a random color
         r = colors[colorIndex][0];
         g = colors[colorIndex][1];
         b = colors[colorIndex][2];
         
         // Initially it's fully alive
         life = 1.0f;
      }
   }

   @Override
   public void keyPressed(KeyEvent e) {
      switch (e.getKeyCode()) {
         case VK_T:
            if (!enabledBurst) enabledBurst = true;
            break;
      }
   }

   @Override
   public void keyReleased(KeyEvent e) {}

   @Override
   public void keyTyped(KeyEvent e) {}
}
